@@ -12,9 +12,9 @@
 #' @param type a label of the weight type:
 #' \describe{
 #'  \item{\code{"frequency"}}{integer feature count (default when a dfm is created)}
-#'  \item{\code{"relFreq"}}{the proportion of the feature counts of total feature counts (aka relative frequency)}
-#'  \item{\code{"relMaxFreq"}}{the proportion of the feature counts of the highest feature count in a document}
-#'  \item{\code{"logFreq"}}{take the logarithm of 1 + the feature count, for base 10}
+#'  \item{\code{"relfreq"}}{the proportion of the feature counts of total feature counts (aka relative frequency)}
+#'  \item{\code{"relmaxfreq"}}{the proportion of the feature counts of the highest feature count in a document}
+#'  \item{\code{"logfreq"}}{take the logarithm of 1 + the feature count, for base 10}
 #'  \item{\code{"tfidf"}}{Term-frequency * inverse document frequency. For a
 #'   full explanation, see, for example, 
 #'   \url{http://nlp.stanford.edu/IR-book/html/htmledition/term-frequency-and-weighting-1.html}.
@@ -28,7 +28,7 @@
 #'   for the corresponding named fatures.  Any features not named will be 
 #'   assigned a weight of 1.0 (meaning they will be unchanged).
 #' @note For finer grained control, consider calling the convenience functions directly.
-#' @return The dfm with weighted values.
+#' @return \code{dfm_weight} returns the dfm with weighted values.
 #' @export
 #' @seealso \code{\link{tf}},  \code{\link{tfidf}}, \code{\link{docfreq}}
 #' @keywords dfm
@@ -38,19 +38,18 @@
 #' 
 #' x <- apply(dtm, 1, function(tf) tf/max(tf))
 #' topfeatures(dtm)
-#' normDtm <- dfm_weight(dtm, "relFreq")
+#' normDtm <- dfm_weight(dtm, "relfreq")
 #' topfeatures(normDtm)
-#' maxTfDtm <- dfm_weight(dtm, type = "relMaxFreq")
+#' maxTfDtm <- dfm_weight(dtm, type = "relmaxfreq")
 #' topfeatures(maxTfDtm)
-#' logTfDtm <- dfm_weight(dtm, type = "logFreq")
+#' logTfDtm <- dfm_weight(dtm, type = "logfreq")
 #' topfeatures(logTfDtm)
 #' tfidfDtm <- dfm_weight(dtm, type = "tfidf")
 #' topfeatures(tfidfDtm)
 #' 
 #' # combine these methods for more complex dfm_weightings, e.g. as in Section 6.4
 #' # of Introduction to Information Retrieval
-#' head(logTfDtm <- dfm_weight(dtm, type = "logFreq"))
-#' head(tfidf(logTfDtm, normalize = FALSE))
+#' head(tfidf(dtm, scheme_tf = "log"))
 #' 
 #' #' # apply numeric weights
 #' str <- c("apple is better than banana", "banana banana apple much better")
@@ -59,7 +58,7 @@
 #' 
 #' \dontshow{
 #' testdfm <- dfm(data_corpus_inaugural[1:5])
-#' for (w in c("frequency", "relFreq", "relMaxFreq", "logFreq", "tfidf")) {
+#' for (w in c("frequency", "relfreq", "relmaxfreq", "logfreq", "tfidf")) {
 #'     testw <- dfm_weight(testdfm, w)
 #'     cat("\n\n=== weight() TEST for:", w, "; class:", class(testw), "\n")
 #'     head(testw)
@@ -68,15 +67,16 @@
 #'   \emph{Introduction to Information Retrieval}. Vol. 1. Cambridge: Cambridge 
 #'   University Press, 2008.
 dfm_weight <- function(x, 
-                       type = c("frequency", "relFreq", "relMaxFreq", "logFreq", "tfidf"),
+                       type = c("frequency", "relfreq", "relmaxfreq", "logfreq", "tfidf"),
                        weights = NULL) {
     UseMethod("dfm_weight")
 }
 
-
-dfm_weight <- function(x, 
-                       type = c("frequency", "relFreq", "relMaxFreq", "logFreq", "tfidf"),
-                       weights = NULL) {
+#' @noRd
+#' @export
+dfm_weight.dfm <- function(x, 
+                           type = c("frequency", "relfreq", "relmaxfreq", "logfreq", "tfidf"),
+                           weights = NULL) {
     # for numeric weights
     if (!is.null(weights)) {
         if (!missing(type)) warning("type is ignored when numeric weights are supplied")
@@ -96,15 +96,16 @@ dfm_weight <- function(x,
         
     } else {
         # named type weights
+        type <- char_tolower(type)
         type <- match.arg(type)
         
         if (x@weightTf[["scheme"]] != "count") {
             catm("  No weighting applied: you should not weight an already weighted dfm.\n")
-        } else if (type=="relFreq") {
+        } else if (type=="relfreq") {
             return(tf(x, "prop"))
-        } else if (type=="relMaxFreq") {
+        } else if (type=="relmaxfreq") {
             return(tf(x, "propmax"))
-        } else if (type=="logFreq") {
+        } else if (type=="logfreq") {
             return(tf(x, "log"))
         } else if (type=="tfidf") {
             return(tfidf(x))
@@ -118,15 +119,21 @@ dfm_weight <- function(x,
 
 #' @rdname dfm_weight
 #' @param smoothing constant added to the dfm cells for smoothing, default is 1
-#' @details This converts a matrix from sparse to dense format, so may exceed memory
-#' requirements depending on the size of your input matrix.
+#' @return \code{dfm_smooth} returns a dfm whose values have been smoothed by
+#'   adding the \code{smoothing} amount. Note that this effectively converts a
+#'   matrix from sparse to dense format, so may exceed memory requirements
+#'   depending on the size of your input matrix.
 #' @export
 #' @examples 
 #' # smooth the dfm
 #' dfm_smooth(mydfm, 0.5)
 dfm_smooth <- function(x, smoothing = 1) {
-    if (!is.dfm(x))
-        stop("x must be a dfm object")
+    UseMethod("dfm_smooth")
+}
+
+#' @noRd
+#' @export
+dfm_smooth.dfm <- function(x, smoothing = 1) {
     x + smoothing
 }
 
@@ -180,10 +187,14 @@ dfm_smooth <- function(x, smoothing = 1) {
 #'   \emph{Introduction to Information Retrieval}. Cambridge University Press.
 docfreq <- function(x, scheme = c("count", "inverse", "inversemax", "inverseprob", "unary"),
                     smoothing = 0, k = 0, base = 10, threshold = 0, USE.NAMES = TRUE) {
-    
-    if (!is.dfm(x))
-        stop("x must be a dfm object")
-    
+    UseMethod("docfreq")
+}
+
+#' @noRd
+#' @export
+docfreq.dfm <- function(x, scheme = c("count", "inverse", "inversemax", "inverseprob", "unary"),
+                        smoothing = 0, k = 0, base = 10, threshold = 0, USE.NAMES = TRUE) {
+
     scheme <- match.arg(scheme)
     args <- as.list(match.call(expand.dots=FALSE))
     if ("base" %in% names(args) & !(substring(scheme, 1, 7) == "inverse"))
@@ -204,9 +215,10 @@ docfreq <- function(x, scheme = c("count", "inverse", "inversemax", "inverseprob
         
     } else if (scheme == "count") {
         if (is(x, "dfmSparse")) {
-            tx <- t(x)
-            featfactor <- factor(tx@i, 0:(nfeature(x)-1), labels = featnames(x))
-            result <- as.integer(table(featfactor[tx@x > threshold]))
+            result <- colSums(x > threshold)
+            # tx <- t(x)
+            # featfactor <- factor(tx@i, 0:(nfeature(x)-1), labels = featnames(x))
+            # result <- as.integer(table(featfactor[tx@x > threshold]))
         } else {
             if (!any(x@x <= threshold)) 
                 result <- rep(ndoc(x), nfeature(x))
@@ -224,10 +236,10 @@ docfreq <- function(x, scheme = c("count", "inverse", "inversemax", "inverseprob
     } else if (scheme == "inverseprob") {
         dftmp <- docfreq(x, "count", USE.NAMES = FALSE)
         result <- log((ndoc(x) - dftmp) / (k + dftmp), base = base)
-        result[is.infinite(result)] <- 0
+        result <- pmax(0, result)
     }
     
-    if (USE.NAMES) names(result) <- featnames(x)
+    if (USE.NAMES) names(result) <- featnames(x) else names(result) <- NULL
     result
 }
 
@@ -235,26 +247,29 @@ docfreq <- function(x, scheme = c("count", "inverse", "inversemax", "inverseprob
 
 #' compute tf-idf weights from a dfm
 #' 
-#' Compute tf-idf, inverse document frequency, and relative term frequency on 
-#' document-feature matrices.  See also \code{\link{weight}}.
+#' Weight a dfm by term frequency-inverse document frequency (tf-idf) using
+#' fully sparse methods.
 #' @param x object for which idf or tf-idf will be computed (a document-feature 
 #'   matrix)
-#' @param normalize if \code{TRUE}, use relative term frequency
-#' @param scheme scheme for \code{\link{docfreq}}
-#' @param ... additional arguments passed to \code{\link{docfreq}} when calling
+#' @param scheme_tf scheme for \code{\link{tf}}; defaults to \code{"count"}
+#' @param scheme_df scheme for \code{link{docfreq}}; defaults to
+#'   \code{"inverse"}
+#' @param base for the logarithms in the \code{tf} and \code{docfreq} calls
+#' @param ... additional arguments passed to \code{\link{docfreq}} when calling 
 #'   \code{tfidf}
 #' @details \code{tfidf} computes term frequency-inverse document frequency 
 #'   weighting.  The default is not to normalize term frequency (by computing 
 #'   relative term frequency within document) but this will be performed if 
-#'   \code{normalize = TRUE}.  
+#'   \code{scheme_tf = "prop"}.
 #' @references Manning, C. D., Raghavan, P., & Schutze, H. (2008). 
 #'   \emph{Introduction to Information Retrieval}. Cambridge University Press.
+#' @seealso \code{\link{tf}}, \code{\link{docfreq}}
 #' @keywords internal weighting dfm
 #' @examples 
-#' head(data_dfm_LBGexample[, 5:10])
-#' head(tfidf(data_dfm_LBGexample)[, 5:10])
-#' docfreq(data_dfm_LBGexample)[5:15]
-#' head(tf(data_dfm_LBGexample)[, 5:10])
+#' head(data_dfm_lbgexample[, 5:10])
+#' head(tfidf(data_dfm_lbgexample)[, 5:10])
+#' docfreq(data_dfm_lbgexample)[5:15]
+#' head(tf(data_dfm_lbgexample)[, 5:10])
 #' 
 #' # replication of worked example from
 #' # https://en.wikipedia.org/wiki/Tf-idf#Example_of_tf.E2.80.93idf
@@ -268,9 +283,33 @@ docfreq <- function(x, scheme = c("count", "inverse", "inversemax", "inverseprob
 #' tfidf(wikiDfm)
 #' @keywords internal weighting
 #' @export
-tfidf <- function(x, normalize = FALSE, scheme = "inverse", ...) {
-    if (!is.dfm(x))
-        stop("x must be a dfm object")
+tfidf <- function(x, scheme_tf = "prop", scheme_df = "inverse", base = 10, ...) {
+    UseMethod("tfidf")
+}
+
+#' @noRd
+#' @export
+tfidf.dfm <- function(x, scheme_tf = "count", scheme_df = "inverse", base = 10, ...) {
+
+    args <- list(...)
+    if ("normalize" %in% names(args)) {
+        warning("normalize is deprecated; use scheme_tf = \"prop\" instead")
+        scheme_tf <- if (args[["normalize"]] == TRUE) "prop" else "count"
+        return(tfidf(x, scheme_tf = scheme_tf, scheme_df = scheme_df, base = base))
+    }
+
+    dfreq <- docfreq(x, scheme = scheme_df, base = base, ...)
+    tfreq <- tf(x, scheme = scheme_tf, base = base)
+    if (nfeature(x) != length(dfreq)) 
+        stop("missing some values in idf calculation")
+    # get the document indexes
+    j <- as(tfreq, "dgTMatrix")@j + 1
+    # replace just the non-zero values by product with idf
+    x@x <- tfreq@x * dfreq[j]
+    x
+}
+
+tfidf_old <- function(x, normalize = FALSE, scheme = "inverse", ...) {
     invdocfr <- docfreq(x, scheme = scheme, ...)
     if (normalize) x <- tf(x, "prop")
     if (nfeature(x) != length(invdocfr)) 
@@ -279,7 +318,6 @@ tfidf <- function(x, normalize = FALSE, scheme = "inverse", ...) {
 }
 
 
-
 #' compute (weighted) term frequency from a dfm
 #' 
 #' Apply varieties of term frequency weightings to a \link{dfm}.
@@ -317,13 +355,20 @@ tfidf <- function(x, normalize = FALSE, scheme = "inverse", ...) {
 #' @keywords internal weighting dfm
 tf <- function(x, scheme = c("count", "prop", "propmax", "boolean", "log", "augmented", "logave"),
                base = 10, K = 0.5) {
+    UseMethod("tf")
+}
+
+#' @noRd
+#' @export
+tf.dfm <- function(x, scheme = c("count", "prop", "propmax", "boolean", "log", "augmented", "logave"),
+               base = 10, K = 0.5) {
     if (!is.dfm(x))
         stop("x must be a dfm object")
     
     scheme <- match.arg(scheme)
     args <- as.list(match.call(expand.dots=FALSE))
-    if ("base" %in% names(args) & !(scheme %in% c("log", "logave")))
-        warning("base not used for this scheme")
+    # if ("base" %in% names(args) & !(scheme %in% c("log", "logave")))
+    #     warning("base not used for this scheme")
     if ("K" %in% names(args) & scheme != "augmented")
         warning("K not used for this scheme")
     if (K < 0 | K > 1.0)
@@ -366,17 +411,15 @@ tf <- function(x, scheme = c("count", "prop", "propmax", "boolean", "log", "augm
         x@weightTf[["K"]] <- K
         
     } else if (scheme == "logave") {
-        meantf <- Matrix::rowMeans(x)
+        meantf <- Matrix::rowSums(x) / Matrix::rowSums(tf(x, "boolean"))
         if (is(x, "dfmSparse"))
             x@x <- (1 + log(x@x, base)) / (1 + log(meantf[x@i+1], base))
         else
             x <- (1 + log(x, base)) / (1 + log(meantf, base))
         x@weightTf[["base"]] <- base
         
-    } else {
-        stop("shouldn't be here!")
-    }
-    
+    } else stop("invalid tf scheme")
+
     x@weightTf[["scheme"]] <- scheme
     return(x)
 }
