@@ -71,6 +71,17 @@ print_dictionary <- function(entry, level = 1) {
 }
 
 
+# Internal function for special handling of multi-word dicitionary values
+split_dictionary_values <- function(values, concatenator) {
+    if (any(stri_detect_charclass(values, "\\p{Z}"))) {
+        values <- c(phrase(values), as.list(stri_replace_all_charclass(values, "\\p{Z}", concatenator)))
+    } else {
+        values <- as.list(values)
+    }
+    return(values)
+}
+
+
 #' print a dictionary object
 #' 
 #' Print/show method for dictionary objects.
@@ -79,10 +90,10 @@ print_dictionary <- function(entry, level = 1) {
 #' @export
 setMethod("show", "dictionary2", 
           function(object) {
-              levs <- ifelse((depth <- dictionary_depth(object)) > 1, " primary", "")
+              levs <- if ((depth <- dictionary_depth(object)) > 1L) " primary" else ""
               nkeys <- length(names(object))
               cat("Dictionary object with ", nkeys, levs, " key entr", 
-                  ifelse(nkeys == 1, "y", "ies"), sep = "")
+                  if (nkeys == 1L) "y" else "ies", sep = "")
               if (levs != "") cat(" and ", depth, " nested levels", sep = "")
               cat(".\n")
               print_dictionary(object)
@@ -97,7 +108,7 @@ setMethod("[",
           signature = c("dictionary2", i = "index"),
           function(x, i) {
               is_category <- sapply(as.list(x)[i], function(y) !is.null(y))
-              dictionary(as.list(x)[i][is_category], concatenator = x@concatenator)
+              dictionary(as.list(x)[i][is_category])
         })
 
 #' Extractor for dictionary objects
@@ -111,7 +122,7 @@ setMethod("[[",
               if (!is.list(as.list(x)[[i]])) {
                   as.list(x)[[i]]
               } else {
-                  dictionary(as.list(x)[[i]], concatenator = x@concatenator)
+                  dictionary(as.list(x)[[i]])
               }
           })
 
@@ -140,9 +151,8 @@ setMethod("as.list",
 #' LIWC format works with 
 #' all currently available dictionary files supplied as part of the LIWC 2001, 
 #' 2007, and 2015 software (see References).
-#' @param ... a named list of character vector dictionary entries, including \link{valuetype} pattern
-#'  matches, and including multi-word expressions separated by \code{concatenator}.  The argument 
-#'  may be an explicit list or named set of elements that can be turned into a list.  See examples.
+#' @param x a named list of character vector dictionary entries, including \link{valuetype} pattern
+#'  matches, and including multi-word expressions separated by \code{concatenator}.  See examples.
 #'  This argument may be omitted if the dictionary is read from \code{file}.
 #' @param file file identifier for a foreign dictionary
 #' @param format character identifier for the format of the foreign dictionary. 
@@ -153,9 +163,8 @@ setMethod("as.list",
 #'   the Linguistic Inquiry and Word Count software} \item{\code{"yoshikoder"}}{
 #'   format used by Yoshikoder software} \item{\code{"lexicoder"}}{format used
 #'   by Lexicoder} \item{\code{"YAML"}}{the standard YAML format}}
-#' @param concatenator the character in between multi-word dictionary values. 
-#'   This defaults to \code{"_"} except LIWC-formatted files, which defaults to 
-#'   a single space \code{" "}.
+#' @param separator the character in between multi-word dictionary values. 
+#'   This defaults to \code{" "}.
 #' @param encoding additional optional encoding value for reading in imported 
 #'   dictionaries. This uses the \link{iconv} labels for encoding.  See the 
 #'   "Encoding" section of the help for \link{file}.
@@ -186,76 +195,156 @@ setMethod("as.list",
 #'                           country = "america"))
 #' head(dfm(mycorpus, dictionary = mydict))
 #' 
-#' # also works
-#' mydict2 <- dictionary(christmas = c("Christmas", "Santa", "holiday"),
-#'                       opposition = c("Opposition", "reject", "notincorpus"))
-#' dfm(mycorpus, dictionary = mydict2)
-#' 
 #' \dontrun{
-#' # import the Laver-Garry dictionary from http://bit.ly/1FH2nvf
-#' lgdict <- dictionary(file = "http://www.kenbenoit.net/courses/essex2014qta/LaverGarry.cat",
-#'                      format = "wordstat")
+#' # import the Laver-Garry dictionary from Provalis Research
+#' dictfile <- tempfile()
+#' download.file("https://provalisresearch.com/Download/LaverGarry.zip", dictfile, mode = "wb")
+#' unzip(dictfile, exdir = (td <- tempdir()))
+#' lgdict <- dictionary(file = paste(td, "LaverGarry.cat", sep = "/"))
 #' head(dfm(data_corpus_inaugural, dictionary = lgdict))
 #' 
 #' # import a LIWC formatted dictionary from http://www.moralfoundations.org
-#' mfdict <- dictionary(file = "http://ow.ly/VMRkL", format = "LIWC")
-#' head(dfm(data_corpus_inaugural, dictionary = mfdict))}
+#' download.file("https://goo.gl/5gmwXq", tf <- tempfile())
+#' mfdict <- dictionary(file = tf, format = "LIWC")
+#' head(dfm(data_corpus_inaugural, dictionary = mfdict))
+#' }
 #' @export
-dictionary <- function(..., file = NULL, format = NULL, 
-                       concatenator = " ", 
+dictionary <- function(x, file = NULL, format = NULL, 
+                       separator = " ", 
                        tolower = TRUE, encoding = "auto") {
+    UseMethod("dictionary")
+}
+
+# method for when x is not supplied, but file is
+#' @export
+dictionary.default <- function(x, file = NULL, format = NULL, 
+                               separator = " ", 
+                               tolower = TRUE, encoding = "auto") {
+    if (!missing(x) & is.null(file))
+        stop("x must be a list if file is not specified")
     
-    if (is.null(file)) {
-        x <- list(...)
-        if (length(x) == 1 && is.list(x[[1]]) && is.null(names(x))) {
-            x <- x[[1]]
+    formats <- c(cat = "wordstat", dic = "LIWC", ykd = "yoshikoder", lcd = "yoshikoder", 
+                 lc3 = "lexicoder", yml = "YAML")
+    
+    if (!file.exists(file))
+        stop("File does not exist: ", file)
+
+    if (is.null(format)) {
+        ext <- stri_trans_tolower(tools::file_ext(file))
+        if (ext %in% names(formats)) {
+            format <- formats[[ext]]
         } else {
-            x <- x[sapply(x, function(x) is.character(x) || is.list(x))]
-        }
-        if (!is.null(x) && !is.list(x)) {
-            stop("Dictionaries must be named lists or lists of named lists.")
-        }
-        if (is.dictionary(x)) {
-            return(x)
-        } else {
-            # a dictionary is a list of lists in the new implementation
-            x <- list2dictionary(x)
-        }
-    } else { 
-        
-        formats <- c(cat = "wordstat", dic = "LIWC", ykd = "yoshikoder", lcd = "yoshikoder", 
-                     lc3 = "lexicoder", yml = "YAML")
-        
-        if (!file.exists(file))
-            stop("File does not exist: ", file)
-        if (is.null(format)) {
-            ext <- stri_trans_tolower(tools::file_ext(file))
-            if (ext %in% names(formats)) {
-                format <- formats[[ext]]
-            } else {
-                stop("Unknown dictionary file extension: ", ext)
-            }
-        } else {
-            format <- match.arg(format, formats)
-        }
-        
-        if (format == "wordstat") {
-            x <- read_dict_wordstat(file, encoding)
-        } else if (format == "LIWC") {
-            # x <- read_dict_liwc(file, encoding)
-            x <- list2dictionary(read_dict_liwc_old(file, encoding))
-        } else if (format == "yoshikoder") {
-            x <- read_dict_yoshikoder(file)
-        } else if (format == "lexicoder") {
-            x <- read_dict_lexicoder(file)
-        } else if (format == "YAML") {
-            x <- yaml::yaml.load_file(file, as.named.list = TRUE)
-            x <- list2dictionary(x)
+            stop("Unknown dictionary file extension: ", ext)
         }
+    } else {
+        format <- match.arg(format, formats)
     }
+    
+    if (format == "wordstat") {
+        x <- read_dict_wordstat(file, encoding)
+    } else if (format == "LIWC") {
+        x <- read_dict_liwc(file, encoding)
+    } else if (format == "yoshikoder") {
+        x <- read_dict_yoshikoder(file)
+    } else if (format == "lexicoder") {
+        x <- read_dict_lexicoder(file)
+    } else if (format == "YAML") {
+        x <- yaml::yaml.load_file(file, as.named.list = TRUE)
+        x <- list2dictionary(x)
+    }
+    
+    ## this code is repeated from dictionary.list()
+    ## would be better to return a one-level list and send that to 
+    ## dictionary.list.  This means the above functions would be more like the
+    ## yaml return, and we would not call list2dictionary() in dictionary.default().
+    ## The code below is a workaround until you can modify the first four read_*
+    ## functions.  --ken
     if (tolower)
-        x <- lowercase_dictionary(x)
-    new("dictionary2", x, concatenator = concatenator)
+        x <- lowercase_dictionary_values(x)
+    x <- replace_dictionary_values(x, separator, " ")
+    x <- as.list(new("dictionary2", x, concatenator = " "))
+    
+    dictionary(x, separator = separator, tolower = tolower)
+}
+
+#' @export
+dictionary.list <- function(x, file = NULL, format = NULL, 
+                               separator = " ", 
+                               tolower = TRUE, encoding = "auto") {
+    if (!is.null(file) | !is.null(format) | encoding != "auto")
+        stop("cannot specify file, format, or encoding when x is a list")
+    if (!is.character(separator) || stri_length(separator) == 0)
+        stop("separator must be a non-empty character")
+    x <- list2dictionary(x)
+    if (tolower)
+        x <- lowercase_dictionary_values(x)
+    x <- replace_dictionary_values(x, separator, " ")
+    new("dictionary2", x, concatenator = " ") # keep concatenator attributes for compatibility
+}
+
+#' @export
+dictionary.dictionary2 <- function(x, file = NULL, format = NULL, 
+                            separator = " ", 
+                            tolower = TRUE, encoding = "auto") {
+    dictionary(as.list(x), separator = separator, tolower = tolower, encoding = encoding)
+}
+
+#' coercion and checking functions for dictionary objects
+#' 
+#' Convert a dictionary from a different format into a \pkg{quanteda} 
+#' dictionary, or check to see if an object is a dictionary.  
+#' @param x object to be coerced or checked; current legal values are a
+#'   data.frame with the fields \code{word} and \code{sentiment} (as per the 
+#'   \strong{tidytext} package)
+#' @return \code{as.dictionary} returns a \link{dictionary} object.  This conversion
+#' function differs from the \code{\link{dictionary}} constructor function in that it
+#' converts an existing object rather than creates one from components or from a
+#' file.
+#' @export
+#' @examples 
+#' \dontrun{
+#' data(sentiments, package = "tidytext")
+#' as.dictionary(subset(sentiments, lexicon == "nrc"))
+#' as.dictionary(subset(sentiments, lexicon == "bing"))
+#' # to convert AFINN into polarities - adjust thresholds if desired
+#' afinn <- subset(sentiments, lexicon == "AFINN")
+#' afinn[["sentiment"]] <-
+#'     with(afinn,
+#'          sentiment <- ifelse(score < 0, "negative",
+#'                              ifelse(score > 0, "positive", "netural"))
+#'     )
+#' with(afinn, table(score, sentiment))
+#' as.dictionary(afinn)
+#' }
+#' 
+as.dictionary <- function(x) {
+    UseMethod("as.dictionary")
+}
+
+#' @noRd
+#' @method as.dictionary data.frame
+#' @export
+as.dictionary.data.frame <- function(x) {
+    if (!all(c("word", "sentiment") %in% names(x)))
+        stop("data.frame must contain word and sentiment columns")
+    if ("lexicon" %in% names(x) && length(unique(x[["lexicon"]])) > 1)
+        warning("multiple values found in a \'lexicon\' column; you may be mixing different dictionaries")
+    if (all(is.na(x[["sentiment"]])))
+        stop("sentiment values are missing")
+    dictionary(with(x, split(as.character(word), as.character(sentiment))))
+}
+
+#' @rdname as.dictionary
+#' @return \code{is.dictionary} returns \code{TRUE} if an object is a
+#'   \pkg{quanteda} \link{dictionary}.
+#' @export
+#' @examples
+#' is.dictionary(dictionary(list(key1 = c("val1", "val2"), key2 = "val3")))
+#' ## [1] TRUE
+#' is.dictionary(list(key1 = c("val1", "val2"), key2 = "val3"))
+#' ## [1] FALSE
+is.dictionary <- function(x) {
+    is(x, "dictionary2")
 }
 
 
@@ -316,7 +405,7 @@ flatten_dictionary <- function(dict, levels = 1:100, level = 1, key_parent = '',
     return(dict_flat)
 }
 
-# Internal function to lowercase dictionary entries
+# Internal function to lowercase dictionary values
 #
 # hdict <- list(KEY1 = list(SUBKEY1 = c("A", "B"),
 #                           SUBKEY2 = c("C", "D")),
@@ -324,13 +413,13 @@ flatten_dictionary <- function(dict, levels = 1:100, level = 1, key_parent = '',
 #                           SUBKEY4 = c("G", "F", "I")),
 #               KEY3 = list(SUBKEY5 = list(SUBKEY7 = c("J", "K")),
 #                           SUBKEY6 = list(SUBKEY8 = c("L"))))
-# lowercase_dictionary(hdict)
+# lowercase_dictionary_values(hdict)
 #
-lowercase_dictionary <- function(dict) {
+lowercase_dictionary_values <- function(dict) {
     dict <- unclass(dict)
     for (i in seq_along(dict)) {
         if (is.list(dict[[i]])) {
-            dict[[i]] <- lowercase_dictionary(dict[[i]])
+            dict[[i]] <- lowercase_dictionary_values(dict[[i]])
         } else {
             if (is.character(dict[[i]])) {
                 dict[[i]] <- stri_trans_tolower(dict[[i]])
@@ -339,6 +428,28 @@ lowercase_dictionary <- function(dict) {
     }
     return(dict)
 }
+# Internal function to replace dictionary values
+#
+# hdict <- list(KEY1 = list(SUBKEY1 = c("A_B"),
+#                           SUBKEY2 = c("C_D")),
+#               KEY2 = list(SUBKEY3 = c("E_F"),
+#                           SUBKEY4 = c("G_F_I")),
+#               KEY3 = list(SUBKEY5 = list(SUBKEY7 = c("J_K")),
+#                           SUBKEY6 = list(SUBKEY8 = c("L"))))
+# replace_dictionary_values(hdict, '_', ' ')
+replace_dictionary_values <- function(dict, from, to) {
+    dict <- unclass(dict)
+    for (i in seq_along(dict)) {
+        if (is.list(dict[[i]])) {
+            dict[[i]] <- replace_dictionary_values(dict[[i]], from, to)
+        } else {
+            if (is.character(dict[[i]])) {
+                dict[[i]] <- stri_replace_all_fixed(dict[[i]], from, to)
+            }
+        }
+    }
+    return(dict)
+}
 
 # Internal function to convert a list to a dictionary
 list2dictionary <- function(dict) {
@@ -357,16 +468,6 @@ list2dictionary <- function(dict) {
 }
 
 
-#' check if an object is a dictionary
-#' 
-#' Return \code{TRUE} if an object is a \pkg{quanteda} \link{dictionary}.
-#' @param x any object
-#' @export
-is.dictionary <- function(x) {
-    is(x, "dictionary2")
-}
-
-
 # Import a Lexicoder dictionary
 # dict <- read_dict_lexicoder('/home/kohei/Documents/Dictionary/Lexicoder/LSDaug2015/LSD2015.lc3')
 read_dict_lexicoder <- function(path) {
@@ -439,22 +540,66 @@ read_dict_liwc <- function(path, encoding = 'auto') {
     lines <- lines[lines != '']
     
     sections <- which(lines == '%')
+    
+    if (length(sections) < 2) {
+        stop('Start and end of a category legend should be marked by percentage symbols, none found')
+    }
+    
     lines_key <- lines[(sections[1] + 1):(sections[2] - 1)]
     lines_value <- lines[(sections[2] + 1):(length(lines))]
     
-    keys <- stri_extract_last_regex(lines_key, '[^\t]+')
-    keys_id <- stri_extract_first_regex(lines_key, '\\d+')
+    # remove any lines with <of>
+    has_oftag <- stri_detect_fixed(lines_value, '<of>')
+    if (any(has_oftag)) {
+        catm("note: ", sum(has_oftag), " term",
+             if (sum(has_oftag) > 1L) "s" else "", 
+             " ignored because contains unsupported <of> tag\n", sep = "")
+        lines_value <- lines_value[!has_oftag]
+    }
     
+    # note odd parenthetical codes
+    has_paren <- stri_detect_regex(lines_value, '[()]')
+    if (any(has_paren)) {
+        catm("note: ignoring parenthetical expressions in lines:\n")
+        for (i in which(has_paren))
+            catm("  [line ", i + sections[2] + 1, "] ", lines_value[i], "\n", sep = "")
+        lines_value <- stri_replace_all_regex(lines_value, '\\(.+\\)', ' ')
+    }
+    
+    lines_key <- stri_replace_all_regex(lines_key, '\\s+', '\t') # fix wrong delimter
+    keys_id <- as.character(as.integer(stri_extract_first_regex(lines_key, '\\d+')))
+    keys <- stri_extract_last_regex(lines_key, '[^\t]+')
+    
+    lines_value <- stri_replace_all_regex(lines_value, '\\s+', '\t') # fix wrong delimter
     values <- stri_extract_first_regex(lines_value, '[^\t]+')
-    lines_value <- stri_replace_first_regex(lines_value, '[^\t]+\t', '') # for safety
+    lines_value <- stri_replace_first_regex(lines_value, '[^\t]+\t', '') # for robustness
     values_ids <- stri_extract_all_regex(lines_value, '\\d+')
+    values_ids <- lapply(values_ids, as.integer)
     
     keys <- stri_replace_all_regex(keys, '[[:control:]]', '') # clean
     values <- stri_replace_all_regex(values, '[[:control:]]', '') # clean
     
     dict <- split(rep(values, lengths(values_ids)), as.factor(unlist(values_ids, use.names = FALSE)))
-    dict <- dict[order(as.numeric(names(dict)))]
+    dict <- lapply(dict, function(x) sort(unique(x))) # remove duplicated and sort values
+    
+    # check if any keys are empty
+    is_empty <- !(keys_id %in% names(dict))
+    if (any(is_empty)) {
+        message("note: removing empty keys: ", paste(keys[is_empty], collapse = ", "))
+    }
+    
+    # check if all categories are defined
+    is_undef <- !(names(dict) %in% keys_id)
+    if (any(is_undef)) {
+        catm("note: ignoring undefined categories:\n")
+        for (i in which(is_undef))
+            catm("  ", names(dict[i]), " for ", dict[[i]], "\n", sep = "")
+        dict <- dict[!is_undef]
+    }
+
+    dict <- dict[order(names(dict))]
     names(dict) <- keys[match(names(dict), keys_id)]
+    
     dict <- list2dictionary(dict)
     return(dict)
     
@@ -465,7 +610,7 @@ read_dict_liwc <- function(path, encoding = 'auto') {
 read_dict_yoshikoder <- function(path){
     
     xml <- XML::xmlParse(path)
-    root <- XML::xpathSApply(xml, "/dictionary/cnode")
+    root <- XML::xpathSApply(xml, "/dictionary")
     dict <- nodes2list(root[[1]])
     dict <- list2dictionary(dict)
     return(dict)
@@ -477,7 +622,7 @@ nodes2list <- function(node, dict = list()){
     if (length(nodes)) {
         for (i in seq_along(nodes)) {
             key <- XML::xmlGetAttr(nodes[[i]], name = "name")
-            dict[[key]] <- nodes2list(nodes[[i]], dict[[key]])
+            dict[[key]] <- nodes2list(nodes[[i]], dict)
         }
     } else {
         dict <- unname(XML::xpathSApply(node, "pnode/@name"))
@@ -491,22 +636,29 @@ nodes2list <- function(node, dict = list()){
 #' \link{dictionary} function into the YAML format. The YAML 
 #' files can be editied in text editors and imported into 
 #' \pkg{quanteda} again.
-#' @param x dictionary object
-#' @return \code{as.yaml} returns a dictionary in the YAML format
+#' @param x a \link{dictionary} object
+#' @return \code{as.yaml} a dictionary in the YAML format, as a character object
 #' @export
 #' @examples
 #' \dontrun{
-#' dict <- dictionary(file = '/home/kohei/Documents/Dictionary/LaverGarry.txt', format = 'wordstat')
-#' yaml <- as.yaml(dict)
-#' cat(yaml, file = '/home/kohei/Documents/Dictionary/LaverGarry.yaml')
+#' dict <- dictionary(list(one = c("a b", "c*"), two = c("x", "y", "z??")))
+#' cat(yaml <- as.yaml(dict))
+#' cat(yaml, file = (yamlfile <- paste0(tempfile(), ".yml")))
+#' dictionary(file = yamlfile)
 #' }
 as.yaml <- function(x) {
+    UseMethod("as.yaml")
+}
+
+#' @noRd
+#' @method as.yaml dictionary2
+#' @export
+as.yaml.dictionary2 <- function(x) {
     yaml <- yaml::as.yaml(simplify_dictionary(x, TRUE), indent.mapping.sequence = TRUE)
     yaml <- stri_enc_toutf8(yaml)
     return(yaml)
 }
 
-
 # Internal function for as.yaml to simplify dictionary objects
 simplify_dictionary <- function(entry, omit = TRUE, dict = list()) {
     entry <- unclass(entry)
@@ -531,12 +683,17 @@ simplify_dictionary <- function(entry, omit = TRUE, dict = list()) {
 
 # return the nested depth of a dictionary
 # a dictionary with no nesting would have a depth of 1
-dictionary_depth <- function(this, thisdepth = -1) {
+dictionary_depth <- function(dict, depth = -1) {
     # http://stackoverflow.com/a/13433689/1270695
-    this <- unclass(this)
-    if (!is.list(this)) {
-        return(thisdepth)
+    dict <- unclass(dict)
+    if (!is.list(dict)) {
+        return(depth)
     } else {
-        return(max(unlist(lapply(this, dictionary_depth, thisdepth = thisdepth + 1))))    
+        return(max(unlist(lapply(dict, dictionary_depth, depth = depth + 1))))    
     }
 }
+
+# check if dictionary has multi-word entries
+has_multiword <- function(dict) {
+    any(stri_detect_fixed(unlist(dict, use.names = FALSE), attr(dict, 'concatenator')))
+}
