@@ -47,7 +47,7 @@ summary(myCorpus, showmeta = TRUE)
 #  # csv file
 #  write.csv(data.frame(inaugSpeech = texts(data_corpus_inaugural),
 #                       docvars(data_corpus_inaugural)),
-#            file = "/tmp/inaug_texts.csv", row.names = FALSE)b
+#            file = "/tmp/inaug_texts.csv", row.names = FALSE)
 #  mytf7 <- readtext("/tmp/inaug_texts.csv", textfield = "inaugSpeech")
 #  summary(corpus(mytf7), 5)
 
@@ -60,11 +60,11 @@ summary(data_corpus_irishbudget2010)
 ## ---- fig.width = 8------------------------------------------------------
 tokenInfo <- summary(data_corpus_inaugural)
 if (require(ggplot2))
-    ggplot(data=tokenInfo, aes(x=Year, y=Tokens, group=1)) + geom_line() + geom_point() +
-        scale_x_discrete(labels=c(seq(1789,2012,12)), breaks=seq(1789,2012,12) ) 
+    ggplot(data=tokenInfo, aes(x = Year, y = Tokens, group = 1)) + geom_line() + geom_point() +
+        scale_x_discrete(labels = c(seq(1789,2012,12)), breaks = seq(1789,2012,12) ) 
 
 # Longest inaugural address: William Henry Harrison
-tokenInfo[which.max(tokenInfo$Tokens),] 
+tokenInfo[which.max(tokenInfo$Tokens), ] 
 
 ## ------------------------------------------------------------------------
 library(quanteda)
@@ -94,9 +94,9 @@ metacorpus(data_corpus_inaugural)
 txt <- c(text1 = "This is $10 in 999 different ways,\n up and down; left and right!", 
          text2 = "@kenbenoit working: on #quanteda 2day\t4ever, http://textasdata.com?page=123.")
 tokens(txt)
-tokens(txt, remove_numbers = TRUE, remove_punct = TRUE)
+tokens(txt, remove_numbers = TRUE,  remove_punct = TRUE)
 tokens(txt, remove_numbers = FALSE, remove_punct = TRUE)
-tokens(txt, remove_numbers = TRUE, remove_punct = FALSE)
+tokens(txt, remove_numbers = TRUE,  remove_punct = FALSE)
 tokens(txt, remove_numbers = FALSE, remove_punct = FALSE)
 tokens(txt, remove_numbers = FALSE, remove_punct = FALSE, remove_separators = FALSE)
 
@@ -130,8 +130,7 @@ head(stopwords("russian"), 10)
 head(stopwords("arabic"), 10)
 
 ## ----warning=FALSE, fig.width = 8, fig.height = 8-----------------------------------------------------------------------------------------------------------------------------------------------------
-mydfm <- dfm(data_char_ukimmig2010, remove = c("will", stopwords("english")), 
-             remove_punct = TRUE)
+mydfm <- dfm(data_char_ukimmig2010, remove = stopwords("english"), remove_punct = TRUE)
 mydfm
 
 ## -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
@@ -167,22 +166,21 @@ byPresMat
 #  liwcdfm[, 1:10]
 
 ## ----fig.width = 6------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-presDfm <- dfm(corpus_subset(data_corpus_inaugural, Year>1980), 
-               remove = stopwords("english"),
-               stem = TRUE, remove_punct = TRUE)
+presDfm <- dfm(corpus_subset(data_corpus_inaugural, Year > 1980), 
+               remove = stopwords("english"), stem = TRUE, remove_punct = TRUE)
 obamaSimil <- textstat_simil(presDfm, c("2009-Obama" , "2013-Obama"), 
                              margin = "documents", method = "cosine")
 obamaSimil
 # dotchart(as.list(obamaSimil)$"2009-Obama", xlab = "Cosine similarity")
 
 ## ---- fig.width = 10, fig.height = 7, eval = FALSE----------------------------------------------------------------------------------------------------------------------------------------------------
-#  data(data_corpus_SOTU, package="quantedaData")
+#  data(data_corpus_SOTU, package = "quantedaData")
 #  presDfm <- dfm(corpus_subset(data_corpus_SOTU, Date > as.Date("1980-01-01")),
-#                 verbose = FALSE, stem = TRUE, remove_punct = TRUE,
-#                 remove = c("will", stopwords("english")))
+#                 stem = TRUE, remove_punct = TRUE,
+#                 remove = stopwords("english"))
 #  presDfm <- dfm_trim(presDfm, min_count = 5, min_docfreq = 3)
 #  # hierarchical clustering - get distances on normalized dfm
-#  presDistMat <- textstat_dist(dfm_weight(presDfm, "relFreq"))
+#  presDistMat <- textstat_dist(dfm_weight(presDfm, "relfreq"))
 #  # hiarchical clustering the distance object
 #  presCluster <- hclust(presDistMat)
 #  # label with document names
@@ -192,20 +190,21 @@ obamaSimil
 
 ## -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 sim <- textstat_simil(presDfm, c("fair", "health", "terror"), method = "cosine", margin = "features")
-as.list(sim)
+lapply(as.list(sim), head, 10)
 
 ## -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 # make prettier document names
 ieDfm <- dfm(data_corpus_irishbudget2010)
-textmodel(ieDfm, model = "wordfish", dir=c(2,1))
+textmodel(ieDfm, model = "wordfish", dir = c(2, 1))
 
 ## -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-quantdfm <- dfm(data_corpus_irishbudget2010,
-                remove = c("will", stopwords("english")))
+quantdfm <- dfm(data_corpus_irishbudget2010, 
+                remove_punct = TRUE, remove_numbers = TRUE, remove = stopwords("english"))
+quantdfm <- dfm_trim(quantdfm, min_count = 4, max_docfreq = 10, verbose = TRUE)
+quantdfm
 
 if (require(topicmodels)) {
     myLDAfit20 <- LDA(convert(quantdfm, to = "topicmodels"), k = 20)
     get_terms(myLDAfit20, 5)
-    topics(myLDAfit20, 3)
 }
 
