@@ -110,7 +110,7 @@ List qatd_cpp_tokens_match(const List &texts_,
     }
 #endif
     // dev::stop_timer("Token match", timer);
-    return recompile(texts, types);
+    return recompile(texts, types, true, true, is_encoded(types_));
 }
 
 /***R
