@@ -128,52 +128,65 @@ test_that("longer selection than longer than features that exist (related to #44
     feat <- c('b', 'c', 'd', 'e', 'f', 'g')
     # bugs in C++ needs repeated tests
     expect_message(dfm_select(dfmtest, feat, verbose = TRUE),
-                   "dfm_select kept 4 features and 2 documents*")
+                   "kept 4 features")
+    expect_message(dfm_remove(dfmtest, feat, verbose = TRUE),
+                   "removed 4 features")
     expect_equivalent(
         as.matrix(dfm_select(dfmtest, feat)),
         matrix(c(1, 1, 0, 1, 0, 1, 0, 1), nrow = 2)
     )
 })
 
-
 test_that("test dfm_select with ngrams #589", {
-    
     ngramdfm <- dfm(c('of_the', 'in_the', 'to_the', 'of_our', 'and_the', ' it_is', 'by_the', 'for_the'))
-    expect_equal(featnames(dfm_select(ngramdfm, features = c('of_the', 'in_the'), valuetype = 'fixed')),
+    expect_equal(featnames(dfm_select(ngramdfm, pattern = c('of_the', 'in_the'), valuetype = 'fixed')),
                  c('of_the', 'in_the'))
-    expect_equal(featnames(dfm_select(ngramdfm, features = '*_the', valuetype = 'glob')),
+    expect_equal(featnames(dfm_select(ngramdfm, pattern = '*_the', valuetype = 'glob')),
                  c('of_the', 'in_the', 'to_the', 'and_the', 'by_the', 'for_the'))
-    
 })
 
-test_that("test dfm_select with documents", {
-    
-    expect_equal(docnames(dfm_select(testdfm, documents = 'doc*', selection = "keep", valuetype = "glob", 
-                                     padding = FALSE, verbose = FALSE, case_insensitive = FALSE)),
-                 c('doc1', 'doc2', 'doc3'))
-    
-    expect_equal(docnames(dfm_select(testdfm, documents = c('doc1', 'doc2'), selection = "keep", valuetype = "fixed", 
-                                    padding = FALSE, verbose = FALSE, case_insensitive = FALSE)),
-                 c('doc1', 'doc2'))
-    
-    expect_equal(docnames(dfm_select(testdfm, documents = c('doc1', 'doc2'), selection = "remove", valuetype = "fixed", 
-                                     padding = FALSE, verbose = FALSE, case_insensitive = FALSE)),
-                 c('doc3'))
-    
-    expect_equal(docnames(dfm_select(testdfm, documents = c('doc3', 'doc4'), selection = "keep", valuetype = "fixed", 
-                                     padding = TRUE, verbose = FALSE, case_insensitive = FALSE)),
-                 c('doc3', 'doc4'))
+test_that("test dfm_select with ngrams concatenated with whitespace", {
+    ngramdfm <- dfm(c('of_the', 'in_the', 'to_the', 'of_our', 'and_the', ' it_is', 'by_the', 'for_the'))
+    colnames(ngramdfm) <- stringi::stri_replace_all_fixed(colnames(ngramdfm), "_", " ")
+    expect_equal(
+        featnames(dfm_select(ngramdfm, pattern = c('of the', 'in the'), valuetype = 'fixed')),
+        c('of the', 'in the')
+    )
+    expect_equal(
+        featnames(dfm_select(ngramdfm, pattern = '* the', valuetype = 'glob')),
+        c('of the', 'in the', 'to the', 'and the', 'by the', 'for the')
+    )
 })
 
-test_that("test dfm_select with dates in document names", {
-    
-    dates <- as.character(seq.Date(as.Date('2017-01-01'), as.Date('2017-12-31'), 1))
-    txts <- paste('text', seq_along(dates))
-    names(txts) <- dates
-    paddeddfm <- dfm_select(dfm(txts)[1:10,], documents = dates, valuetype = 'fixed', padding = TRUE)
-    expect_equal(ndoc(paddeddfm), 365)
-    
-})
+
+# test_that("test dfm_select with documents", {
+#     
+#     expect_equal(docnames(dfm_select(testdfm, documents = 'doc*', selection = "keep", valuetype = "glob", 
+#                                      padding = FALSE, verbose = FALSE, case_insensitive = FALSE)),
+#                  c('doc1', 'doc2', 'doc3'))
+#     
+#     expect_equal(docnames(dfm_select(testdfm, documents = c('doc1', 'doc2'), selection = "keep", valuetype = "fixed", 
+#                                     padding = FALSE, verbose = FALSE, case_insensitive = FALSE)),
+#                  c('doc1', 'doc2'))
+#     
+#     expect_equal(docnames(dfm_select(testdfm, documents = c('doc1', 'doc2'), selection = "remove", valuetype = "fixed", 
+#                                      padding = FALSE, verbose = FALSE, case_insensitive = FALSE)),
+#                  c('doc3'))
+#     
+#     expect_equal(docnames(dfm_select(testdfm, documents = c('doc3', 'doc4'), selection = "keep", valuetype = "fixed", 
+#                                      padding = TRUE, verbose = FALSE, case_insensitive = FALSE)),
+#                  c('doc3', 'doc4'))
+# })
+# 
+# test_that("test dfm_select with dates in document names", {
+#     
+#     dates <- as.character(seq.Date(as.Date('2017-01-01'), as.Date('2017-12-31'), 1))
+#     txts <- paste('text', seq_along(dates))
+#     names(txts) <- dates
+#     paddeddfm <- dfm_select(dfm(txts)[1:10,], documents = dates, valuetype = 'fixed', padding = TRUE)
+#     expect_equal(ndoc(paddeddfm), 365)
+#     
+# })
 
 
 test_that("test dfm_select with features from a dfm,  fixed", {
@@ -197,7 +210,7 @@ test_that("test dfm_select with features from a dfm,  fixed", {
 })
 
 
-test_that("dfm_select returns equal feature sets", {
+test_that("dfm_select on a dfm returns equal feature sets", {
     txts <- c(d1 = "This is text one", d2 = "The second text", d3 = "This is text three")
     dfm1 <- dfm(txts[1:2])
     dfm2 <- dfm(txts[2:3])
@@ -217,35 +230,78 @@ test_that("dfm_select removes padding", {
     
 })
 
-test_that("dfm_select raises warning when padding = TRUE but not valuetype = fixed", {
-    
-  expect_warning(dfm_select(testdfm, c('z', 'd', 'e'), padding = TRUE),
-                 "padding is used only when valuetype is 'fixed'")
-    
-})
+# test_that("dfm_select raises warning when padding = TRUE but not valuetype = fixed", {
+#     
+#   expect_warning(dfm_select(testdfm, c('z', 'd', 'e'), padding = TRUE),
+#                  "padding is used only when valuetype is 'fixed'")
+#     
+# })
 
-test_that("dfm_select return empty dfm when not maching features", {
-    
-    expect_equal(dim(dfm_select(testdfm, features = c('x', 'y', 'z'), documents = 'doc4')),
-                 c(0, 0))
-    
-    expect_equal(print(dfm_select(testdfm, features = c('x', 'y', 'z'))),
-                 NULL)
+test_that("dfm_select returns empty dfm when not maching features", {
+    expect_equal(dim(dfm_select(testdfm, pattern = c('x', 'y', 'z'))),
+                 c(3, 0))
 })
 
 test_that("dfm_remove works even when it does not remove anything, issue 711", {
-  
     txts <- c(d1 = "This is text one", d2 = "The second text", d3 = "This is text three")
     testdfm <- dfm(txts)
     
     expect_silent(dfm_remove(testdfm, c('xxx', 'yyy', 'x y')))
     expect_equal(featnames(dfm_remove(testdfm, c('xxx', 'yyy', 'x y'))),
                  featnames(testdfm))
+})
 
+test_that("dfm_select errors when dictionary has multi-word features, issue 775", {
+    dfm_inaug <- dfm(data_corpus_inaugural[50:58])
+    testdict1 <- dictionary(list(eco = c("compan*", "factory worker*"),
+                                 pol = c("political part*", "election*")),
+                            separator = " ")
+    testdict2 <- dictionary(list(eco = c("compan*", "factory_worker"),
+                                 pol = c("political_part*", "election*")), 
+                            separator = "_")
+    expect_equal(
+        featnames(dfm_select(dfm_inaug, pattern = testdict1, valuetype = "glob")),
+        c("election", "elections", "company", "companies")
+    )
+    expect_equal(
+        featnames(dfm_select(dfm_inaug, pattern = phrase(testdict1), valuetype = "glob")),
+        c("political", "election", "part", "parties", "elections", "partisan", "company", "participation", "party", "partisanship", "partial", "companies")    
+    )
+    expect_equal(
+        featnames(dfm_select(dfm_inaug, pattern = testdict2, valuetype = "glob")),
+        c("election", "elections", "company", "companies")
+    )
+    expect_equal(
+        featnames(dfm_select(dfm_inaug, pattern = phrase(testdict2), valuetype = "glob")),
+        c("political", "election", "part", "parties", "elections", "partisan", "company", "participation", "party", "partisanship", "partial", "companies")    
+    )
 })
 
 
-
-
+test_that("dfm_select works when selecting on collocations", {
+    txt <- c(d1 = "a b c d e g h",  d2 = "a b e g h i j")
+    toks_uni <- tokens(txt)
+    dfm_uni <- dfm(toks_uni)
+    toks_bi <- tokens(txt, n = 2, concatenator = " ")
+    dfm_bi <- dfm(toks_bi)
+    coll_bi <- textstat_collocations(toks_uni, size = 2, min_count = 2)
+    coll_tri <- textstat_collocations(toks_uni, size = 3, min_count = 2)
+    
+    expect_equal(
+        dim(dfm_select(dfm_uni, coll_bi)),
+        c(2, 0)
+    )
+    expect_equal(
+        dim(dfm_select(dfm_uni, coll_tri)),
+        c(2, 0)
+    )
+    
+    expect_equal(sum(dfm_select(dfm_bi, coll_bi)), 6)
+    expect_equal(featnames(dfm_select(dfm_bi, coll_bi)), c("a b", "e g", "g h"))
+    
+    # wrong
+    expect_equal(dim(dfm_select(dfm_bi, coll_tri)), c(2,0))
+    expect_equal(featnames(dfm_select(dfm_bi, coll_tri)), character())
+})
 
 
