@@ -299,3 +299,14 @@ test_that("fcm expects warning for a wrong weights length", {
     expect_warning(fcm(tokens(xts), context = "window", window = 2, count = "weighted", weights = c(1,2,3)),
                  "weights length is not equal to the window size, weights are assigned by default!")
 })
+
+
+test_that("fcm works tokens with paddings, #788", {
+    
+    txt <- c("The quick brown fox jumped over the lazy dog.",
+             "The dog jumped and ate the fox.")
+    toks <- tokens(txt, remove_punct = TRUE)
+    toks <- tokens_remove(toks, pattern = stopwords(), padding = TRUE)
+    testfcm <- fcm(toks, context = "window", window = 3)
+    expect_equal(sort(colnames(testfcm)), sort(attr(toks, 'types')))
+})
